//
//  TestView.swift
//  flutterpluginlcytest
//
//  Created by liucy on 2020/3/25.
//

import UIKit
import SnapKit
import AVFoundation
import Toast_Swift

class EhsureScannerView: NSObject,FlutterPlatformView {
    
        let frambeC: CGRect;
        let viewId: Int64;
        var messenger: FlutterBinaryMessenger!
        var channel: FlutterMethodChannel!
        init(_ frame: CGRect,viewID: Int64,args :Any?, binaryMessenger: FlutterBinaryMessenger) {
            self.frambeC = frame;
            self.viewId = viewID;
            self.messenger=binaryMessenger;
            self.channel = FlutterMethodChannel.init(name: "ehsure_scanner",binaryMessenger: messenger);
        }

    
    func view() -> UIView {

        let vc = ScanVC.init(frame: frambeC) { (code) in
            self.channel?.invokeMethod("scanResult", arguments: code)
        }
        vc.view.frame = frambeC;
//        vc.view.layoutIfNeeded()
        return vc.view
    }
}


enum ScanerType {
    case single//单次扫描, 扫描完后直接pop 带回结果
//    case multi(popVC: ScanVCPopRefreshProtocol)//连续扫描, 手动点完成pop回页面
}
// swiftlint:disable all
class ScanVC: UIViewController, UIImagePickerControllerDelegate,UINavigationControllerDelegate{
        
    //相机显示视图
    let cameraView = UIView()
    
    //创建AVCaptureSession
    let captureSession = AVCaptureSession()
    
    //屏幕扫描区域视图
    let barcodeView = ScanerView()
    
    //扫描线
    let scanLine = UIImageView()
    
    
    var timer = Timer()
    
    //手电筒
    let flashlightBtn = UIButton()
    
    //手电筒显示状态标识
    var isLightOn = true
    
    var callBack: ((String)-> ())!
    
    var isBarcode = true
    
//    var popVC: ScanVCPopRefreshProtocol?
    var scanerType: ScanerType = .single
    
    var selfFrame: CGRect!
    ///
    ///
    /// - Parameters:
    ///   - isBarcode: 是条码 还是确权码 false代表确权码
    ///   - callBack:
    convenience init(isBarcode: Bool = true, type: ScanerType = .single, frame: CGRect, callBack: @escaping (String)-> ()) {
        self.init()
        self.callBack = callBack
        self.isBarcode = isBarcode
        self.scanerType = type
    }
    
    //手动输入
    @objc func tapInput() {
//         let alertVC = AlertStyle7ViewController(nibName: "AlertStyle7ViewController", bundle: nil)
//        let popup = PopupDialog(viewController: alertVC,
//                                buttonAlignment: .horizontal,
//                                transitionStyle: .bounceUp,
//                                tapGestureDismissal: false,
//                                panGestureDismissal: false)
//        let buttonOne = CancelButton(title: "取消", height: 50, dismissOnTap: true, action: nil)
//        let buttonTwo = DefaultButton(title: "确定", height: 50, dismissOnTap: true, action: {
//            let code = alertVC.codeField.text
//            self.callBack(code!)
//            //返回
//            self.navigationController?.popViewController(animated: true)
//        })
//        popup.addButtons([buttonOne, buttonTwo])
//        self.present(popup, animated: true, completion: nil)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.title = "扫码"
        flashlightBtn.isUserInteractionEnabled = true
//        let item = UIBarButtonItem(title: "手输",style: UIBarButtonItem.Style.done,target:self,action: #selector(tapInput))
//        item.tintColor = UIColor.primary
//        self.navigationItem.rightBarButtonItem = item
        self.view.addSubview(cameraView)
        cameraView.snp.makeConstraints { (make) in
            make.edges.equalTo(self.view)
        }
        initView()
        
                //            [AVMetadataObject.ObjectType.qr, AVMetadataObject.ObjectType.ean13, AVMetadataObject.ObjectType.ean8, AVMetadataObject.ObjectType.code128]
        //            NSArray(array: [AVMetadataObjectTypeQRCode,AVMetadataObjectTypeEAN13Code,AVMetadataObjectTypeEAN8Code, AVMetadataObjectTypeCode128Code]) as [AnyObject]
        

        
        //手电筒
        
        flashlightBtn.setImage(UIImage(named: "CodeScan.bundle/flashlight"), for: .normal)
        flashlightBtn.addTarget(self, action: #selector(flashlightOnOrDown), for: .touchUpInside)
        self.view.addSubview(flashlightBtn)
        flashlightBtn.snp.makeConstraints { (make) in
            make.width.height.equalTo(25)
            make.centerX.equalTo(self.view)
            make.top.equalTo(self.barcodeView.snp.bottom).offset(20)
        }
        
    }
    
    @objc func flashlightOnOrDown() {
        let device = AVCaptureDevice.default(for: .video)
        
        if device==nil{
            return
        }
        do{
            //锁定设备以便进行手电筒状态修改
            try device?.lockForConfiguration()
            if isLightOn{
                //设置手电筒模式为亮灯（On）
                device?.torchMode = .on
                isLightOn = false
                
                flashlightBtn.setImage(UIImage(named: "flashlight_open"), for: .normal)
                //改变按钮标题
            }else{
                //设置手电筒模式为关灯（Off）
                flashlightBtn.setImage(UIImage(named: "flashlight"), for: .normal)
                device?.torchMode = .off
                isLightOn = true
                //改变按钮标题
            }
            //解锁设备锁定以便其他APP做配置更新
            device?.unlockForConfiguration()
        }catch{
            return
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        
        let captureDevice = AVCaptureDevice.default(for: .video)
        let input :AVCaptureDeviceInput
        
        //创建媒体数据输出流
        let output = AVCaptureMetadataOutput()
        
        //捕捉异常
        do{
            //创建输入流
            input = try AVCaptureDeviceInput(device: captureDevice!)
            //把输入流添加到会话
            captureSession.addInput(input)
            
            //把输出流添加到会话
            captureSession.addOutput(output)
        }catch {
            print("异常")
            let err = error as NSError
            if (err.code == -11852) {//无法使用“后置镜头
                self.view.makeToast( "您没有权限访问相机, 请在设置中开启访问权限")
            }
            return
        }
        
        //创建串行队列
        let dispatchQueue = DispatchQueue(label: "queue", attributes: [])
        
        //设置输出流的代理
        output.setMetadataObjectsDelegate(self, queue: dispatchQueue)
        
        //设置输出媒体的数据类型
        output.metadataObjectTypes = [AVMetadataObject.ObjectType.qr,
                                      AVMetadataObject.ObjectType.ean8,
                                      AVMetadataObject.ObjectType.ean13,
                                      AVMetadataObject.ObjectType.code128,
                                      AVMetadataObject.ObjectType.code39,
                                      AVMetadataObject.ObjectType.upce, AVMetadataObject.ObjectType.code39Mod43, AVMetadataObject.ObjectType.code93, AVMetadataObject.ObjectType.pdf417]

        //创建预览图层
        let videoPreviewLayer = AVCaptureVideoPreviewLayer(session: captureSession)
        
        //设置预览图层的填充方式
        videoPreviewLayer.videoGravity = AVLayerVideoGravity.resizeAspectFill
        
        cameraView.layoutIfNeeded()
        //设置预览图层的frame
        videoPreviewLayer.frame = cameraView.bounds
        
        //将预览图层添加到预览视图上
        cameraView.layer.insertSublayer(videoPreviewLayer, at: 0)
        
        //设置扫描范围
        output.rectOfInterest = CGRect(x: 0.2, y: 0.2, width: 0.6, height: 0.6)
        self.scannerStart()

    }
    
    func scannerStart(){
        captureSession.startRunning()
    }
    
    func scannerStop() {
        captureSession.stopRunning()
    }
    
    
    
    
    //初始化视图
    func initView()  {
        barcodeView.backgroundColor = UIColor.clear

        cameraView.addSubview(barcodeView)
        barcodeView.snp.makeConstraints { (make) in
            make.center.equalTo(self.view)
            make.width.equalTo(self.view.snp.width).multipliedBy(0.6)
            make.height.equalTo(self.view.snp.width).multipliedBy(0.6)
        }
        
        //设置扫描线
        scanLine.frame = CGRect(x: 0, y: 0, width: barcodeView.frame.size.width, height: 3)
        scanLine.contentMode = .center
        let img = UIImage(named: "QRCodeScanLine")
        scanLine.image = img
        
        //添加扫描线图层
        barcodeView.addSubview(scanLine)
        barcodeView.layer.masksToBounds = true
        scanLine.snp.makeConstraints { (make) in
            make.left.top.right.equalTo(barcodeView)
            make.height.equalTo(3)
        }
        self.createBackGroundView()
        
        timer = Timer.scheduledTimer(timeInterval: 2, target: self, selector: #selector(moveScannerLayer(_:)), userInfo: nil, repeats: true)
        timer.fire()
    }
    
    
    
    //创建背景视图
    func  createBackGroundView() {
        let topView = UIView()
        let bottomView = UIView()
        
        let leftView = UIView()
        let rightView = UIView()
        
        topView.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.4)
        bottomView.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.4)
        leftView.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.4)
        rightView.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.4)
        cameraView.addSubview(topView)
        topView.snp.makeConstraints { (make) in
            make.top.left.right.equalTo(cameraView)
            make.bottom.equalTo(barcodeView.snp.top)
        }
        cameraView.addSubview(bottomView)
        bottomView.snp.makeConstraints { (make) in
            make.bottom.left.right.equalTo(cameraView)
            make.top.equalTo(barcodeView.snp.bottom)
        }

        cameraView.addSubview(leftView)
        leftView.snp.makeConstraints { (make) in
            make.left.equalTo(0)
            make.top.bottom.equalTo(barcodeView)
            make.right.equalTo(barcodeView.snp.left)
        }

        cameraView.addSubview(rightView)
        rightView.snp.makeConstraints { (make) in
            make.right.equalTo(0)
            make.top.bottom.equalTo(barcodeView)
            make.left.equalTo(barcodeView.snp.right)
        }
        
        let topLogoView = UIView()
        let label1 = UILabel()
        label1.font = UIFont.systemFont(ofSize: 14)
        label1.textColor = UIColor.white
        label1.text = "将二维码/条码放于框内，即可自动扫描"
        topLogoView.addSubview(label1)
        label1.snp.makeConstraints { (make) in
            make.centerY.equalTo(topLogoView)
            make.left.equalTo(topLogoView)
            make.right.top.bottom.equalTo(topLogoView)
        }


        self.view.addSubview(topLogoView)
        topLogoView.snp.makeConstraints { (make) in
            make.top.equalTo(30)
            make.centerX.equalTo(self.view)
        }
        
    }
    
    //让扫描线滚动
    @objc func moveScannerLayer(_ timer : Timer) {
        scanLine.snp.remakeConstraints{ (make) in
            make.top.equalTo(0)
            make.left.right.equalTo(barcodeView)
            make.height.equalTo(3)
        }
        UIView.animate(withDuration: 2) {
            self.scanLine.snp.remakeConstraints{ (make) in
                make.top.equalTo(self.barcodeView.snp.bottom)
                make.left.right.equalTo(self.barcodeView)
                make.height.equalTo(3)
            }
        }
    }
    
    
    
    //结果处理
    @objc func processResult(_ scanStr: String) {
        callBack(scanStr)
        //返回
        switch self.scanerType {
        case .single:
            self.navigationController?.popViewController(animated: true)
        default:
            break
        }
    }
    
    //视图将要消失的时候执行
    override func viewWillDisappear(_ animated: Bool) {
        self.scannerStop()
    }
}

extension ScanVC: AVCaptureMetadataOutputObjectsDelegate {
    func metadataOutput(_ output: AVCaptureMetadataOutput, didOutput metadataObjects: [AVMetadataObject], from connection: AVCaptureConnection) {
        
        if metadataObjects != nil && metadataObjects.count > 0 {
            let metaData = metadataObjects.first as! AVMetadataMachineReadableCodeObject
//            AudioServicesPlaySystemSound(kSystemSoundID_Vibrate)
            
            let value = metaData.stringValue
            
            self.scannerStop()
            DispatchQueue.main.async(execute: {
                self.processResult(value ?? "")
            })
        }
        
    }

}

class ScanerView: UIView {
    override func draw(_ rect: CGRect) {
        super.draw(rect)
        let lineWidth: CGFloat = 3.0 //线的宽度
        let angleLength: CGFloat = 15.0//四角的长度
        let context = UIGraphicsGetCurrentContext()
        context?.setStrokeColor(red: 0.0/255, green: 210.0/255, blue: 255.0/255, alpha: 1.0)
        context?.setLineWidth(lineWidth)
        //左上角
        context?.move(to: CGPoint(x: 0, y: 0))
        context?.addLine(to: CGPoint(x: angleLength, y: 0))
        
        context?.move(to: CGPoint(x: 0, y: 0))
        context?.addLine(to: CGPoint(x: 0, y: angleLength))
        
        //右上角
        context?.move(to: CGPoint(x: self.frame.width - angleLength, y: 0))
        context?.addLine(to: CGPoint(x: self.frame.width, y: 0))
        context?.addLine(to: CGPoint(x: self.frame.width, y: angleLength))
        
        //左下角
        context?.move(to: CGPoint(x: 0, y: self.frame.height - angleLength))
        context?.addLine(to: CGPoint(x: 0, y: self.frame.height))
        context?.addLine(to: CGPoint(x: angleLength, y: self.frame.height))
        
        //右下角
        context?.move(to: CGPoint(x: self.frame.width, y: self.frame.height - angleLength))
        context?.addLine(to: CGPoint(x: self.frame.width, y: self.frame.height))
        context?.addLine(to: CGPoint(x: self.frame.width - angleLength, y: self.frame.height))
        
        context?.strokePath()
        //
    }
}

//extension ScanVC {
//    override func shouldPopViewController(byBackButtonOrPopGesture byPopGesture: Bool) -> Bool {
//        switch scanerType {
//        case .multi(let popVC):
//            popVC.scanPopRefresh()
//        default:
//            break
//        }
//        return true
//    }
//}

// swiftlint:enable all

